/*

This file is part of jHaushalt.

jHaushalt is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

jHaushalt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with jHaushalt; if not, see <http://www.gnu.org/licenses/>.


(C)opyright 2002-2010 Dr. Lars H. Hahn

*/

package haushalt.auswertung;

import haushalt.auswertung.bloecke.AbstractBlock;
import haushalt.auswertung.bloecke.LeererBlock;
import haushalt.auswertung.bloecke.TabellenBlock;
import haushalt.auswertung.bloecke.TextBlock;
import haushalt.daten.Datenbasis;
import haushalt.daten.Euro;
import haushalt.daten.EinzelKategorie;
import haushalt.daten.zeitraum.AbstractZeitraum;
import haushalt.daten.zeitraum.Jahr;
import haushalt.gui.Haushalt;
import haushalt.gui.TextResource;
import haushalt.gui.generischerdialog.AbstractGDPane;
import haushalt.gui.generischerdialog.EinOderAlleRegisterGDP;
import haushalt.gui.generischerdialog.MehrereKategorienGDP;
import haushalt.gui.generischerdialog.ZeitraumGDP;

import java.awt.Font;
import java.util.ArrayList;

/**
 * @author Dr. Lars H. Hahn
 * @version 2.5/2007.06.04
 * @since 2.0
 */

/*
 * 2007.06.05 Internationalisierung
 * 2004.08.22 Erste Version
 */
public class BAKategorieAusgabe extends AbstractBlockAuswertung {
  private static final boolean DEBUG = false;
  private static final long serialVersionUID = 1L;
  private static final TextResource res = TextResource.get();

  public static final String ueberschrift = res.getString("table_selected_bookings");

  public BAKategorieAusgabe(Haushalt haushalt, Datenbasis db, String name) {
    super(haushalt, db, name);
    AbstractGDPane[] panes = new AbstractGDPane[3];
		panes[0] = new ZeitraumGDP(res.getString("period")+":", new Jahr(2007));
    panes[1] = new EinOderAlleRegisterGDP(res.getString("register")+":", db, null);
    panes[2] = new MehrereKategorienGDP(res.getString("categories")+":", db);
		erzeugeEigenschaften(haushalt.getFrame(), ueberschrift, panes);
  }

  protected String berechneAuswertung(Object[] werte) {
  	AbstractZeitraum zeitraum = (AbstractZeitraum) werte[0];
    String register = (String) werte[1];
    EinzelKategorie[] kategorien = (EinzelKategorie[]) werte[2];

    boolean unterkategorienVerwenden = ((MehrereKategorienGDP)panes[2]).getUnterkategorienVerwenden();
    ArrayList<String[]> buchungen = db.getBuchungen(zeitraum, register, kategorien, unterkategorienVerwenden);
    int anzahl = buchungen.size();
    tabelle = new String[anzahl][4];
    Euro summe = new Euro();
    for(int i=0; i<anzahl; i++) {
      summe.sum(new Euro(buchungen.get(i)[3]));
      tabelle[i] = buchungen.get(i);
    }
    if(DEBUG)
      System.out.println(""+anzahl+" Buchungen im Zeitraum "+zeitraum);

		// Vorhandene Blcke lschen und neu berechnete einfgen
    String titel = res.getString("selected_bookings")+" ("+zeitraum;
    if(register == null)
      titel += " / "+res.getString("all_registers")+")";
    else
      titel += " / "+register+")";
		loescheBloecke();
		AbstractBlock block1 = new TextBlock(titel);
		block1.setFont(new Font(haushalt.getFontname(), Font.BOLD, haushalt.getFontgroesse()+6));
		addDokumentenBlock(block1);
		addDokumentenBlock(new LeererBlock(1));
    if(anzahl == 0) {
      String hinweis = res.getString("no_bookings_with_this_properties");
			AbstractBlock block2 = new TextBlock(hinweis);
			addDokumentenBlock(block2);
    }
    else {
	    final double[] relTabs = {0.0, 12.0, 50.0, 80.0};
      final TabellenBlock.Ausrichtung[] attribute = {
          TabellenBlock.Ausrichtung.LINKS, 
          TabellenBlock.Ausrichtung.LINKS, 
          TabellenBlock.Ausrichtung.LINKS, 
          TabellenBlock.Ausrichtung.RECHTS};
      TabellenBlock block2 = new TabellenBlock(tabelle);
			block2.setFont(new Font(haushalt.getFontname(), Font.PLAIN, haushalt.getFontgroesse()));
	    block2.setRelTabs(relTabs);
	    block2.setLinienFarbe("Wei");
	    block2.setAusrichtung(attribute);
	    addDokumentenBlock(block2);
	    String[][] text = {{"", res.getString("total")+":", "", ""+summe}};
	    TabellenBlock block3 = new TabellenBlock(text);
			block3.setFont(new Font(haushalt.getFontname(), Font.ITALIC, haushalt.getFontgroesse()));
	    block3.setRelTabs(relTabs);
	    block3.setHgFarbe("Grau");
	    block3.setLinienFarbe("Grau");
	    block3.setAusrichtung(attribute);
	    addDokumentenBlock(block3);
    }
    return titel;
  }

}
